#!/usr/bin/python
# .*. coding: utf-8 .*.
#---------------------- Copyright (c) ABB Bomem inc, 2012 -------------------
#                               Source code
# This software is the property of Bomem and should be considered and treated
# as proprietary information.  Refer to the "Source Code License Agreement"
#----------------------------------------------------------------------------

# pylint: disable=invalid-name

import re
import time
import sys
import os

from pymodbus.client.sync import ModbusSerialClient as ModbusClient
from Console import Console


class H2Scan(object):
    """ Contain function to handle H2Scan in cli mode
    """

    def __init__(self, comport, ledcontrol):
        self.H2SCAN_PROMPT = 'H2Scan:'
        self.console = Console()
        self.COMPORT = comport
        self.ledcontrol = ledcontrol

    def open_cli(self):
        """ Open CLI
        """
        self.console.open(com=self.COMPORT, baudrate=19200)
        self.console.ser.flushInput()

    def close_cli(self):
        """ Close CLI
        """
        self.console.close()

    def dump_console(self, filename):
        """ Dump console
        """
        self.console.dump_console(filename)

    def ask_h2scan_prompt(self, nbretry=60):
        """Ask prompt of H2Scan
        """
        print 'Wait H2Scan CLI Prompt... ',
        sys.stdout.flush()
        for _cnt in range(0, nbretry):
            self.console.ser.write('\r\n\033')
            try:
                h2resp = self.console.wait_for_prompt(prompt=self.H2SCAN_PROMPT, timeout=5)
                print 'OK'
                return h2resp
            except:
                pass

        print 'Error: Didn\'t receive prompt'
        raise Exception

    def set_h2scan_advanced(self):
        """ Set h2scan cli in advanced mode
        """
        print 'Setting Advanced Mode on H2Scan... ',
        self.console.ser.write('=scan\r')
        try:
            self.console.wait_for_prompt(prompt=self.H2SCAN_PROMPT, timeout=5)
            print 'advanced mode enable.\n'
        except:
            print 'Error: Setting CLI in advanced Mode.\n'
            self.console.close()
            raise Exception

    def set_h2scan_in_cli_mode(self):
        """ Set H2Scan in CLI mode
        "    The CLI console must be close before calling this function
        """
        print 'Set H2Scan in CLI mode'
        client = ModbusClient(method='rtu', port=self.COMPORT, baudrate=19200, parity='N', Bytesize=8, timeout=1)
        client.connect()
        client.write_register(149, 0, unit=1)
        client.close()

    def get_h2scan_version(self):
        """ Get H2Scan version
        "     Must be in advanded mode
        """
        # Get the version
        self.console.ser.write('d\r')
        resp = self.console.wait_for_prompt(prompt=self.H2SCAN_PROMPT, timeout=5)
        result = re.search('Firmware Rev:(.*)$', resp, re.MULTILINE | re.IGNORECASE)

        if result:
            return result.group(1)
        return ''

    def read_firmware_crc(self, firm_filename):
        """ Validate firmware file
        """
        print 'Try to open firmware file...',
        try:
            firm = open(firm_filename, 'r')
            firm.close()
        except:
            print 'Error: Opening Hex File: ' + firm_filename
            raise

        try:
            filename = os.path.splitext(firm_filename)
            fcrc = open(filename[0] + '.crc', 'r')
            checksum = fcrc.readline().rstrip()
            fcrc.close()
        except:
            print 'Error: Opening CRC File: ' + filename[0] + '.crc'

        print 'File OK\n'
        return checksum

    def set_h2scan_updateFirmware(self, crc, firm_filename):
        """ Update firmware of H2Scan
        """

        try:
            f = open(firm_filename, 'r')

            print 'File Opened.\n'
            print 'Setting H2SCAN in Firmware Update mode...\n'
            print '\tSent CL...',
            self.console.ser.write('CL\r')
            self.console.wait_for_prompt(prompt='crc:', timeout=2)
            print 'OK'
            print '\tEnter Firmware CRC...',
            self.console.ser.write(crc + '\r')
            self.console.wait_for_prompt(prompt='(Y/N)?', timeout=2)
            print 'OK'
            print '\tWait Send File Prompt...',
            self.console.ser.write('y\r')
            self.console.wait_for_prompt(prompt='Ready...', timeout=5)
            # h2resp = self.console.wait_for_prompt(prompt='>', timeout=5)
            print 'OK'
        except:
            self.console.close()
            f.close()
            print 'ERROR: Download Prompt not received :\n'
            return False

        # print os.path.basename(f.name)
        f.seek(0, 0)
        numlines = 0
        for line in f:
            numlines = numlines + 1
        # numlines = sum(1 for line in f)
        f.seek(0, 0)
        sentlines = 0

        try:
            print '\tDownloading File... Total Lines: ' + str(numlines)
            for line in f:
                if (sentlines % 10) == 0:
                    self.ledcontrol.led_on()
                if (sentlines % 10) == 5:
                    self.ledcontrol.led_off()

                self.console.ser.write(line)
                sentlines = sentlines + 1
                if (sentlines % 10) == 0:
                    print '\tSending line: ' + str(sentlines) + ' out of ' + str(numlines) + '...\r',
                    sys.stdout.flush()
                else:
                    time.sleep(10.0 / 1000.0)

            print '\n\tDownloading File...OK\n'
        except:
            print '\nERROR: Error Downloading File @ line#' + str(sentlines) + ' / ' + str(numlines)
            self.console.close()
            f.close()
            return False

        print '\n Flashing Please Wait...',
        sys.stdout.flush()

        try:
            self.console.wait_for_prompt(prompt='complete:', timeout=30)
            print 'Complete...'
            self.console.close()
        except:
            print 'CRC Failed'
            self.console.close()
            f.close()
            return False

        self.ledcontrol.led_on()

        for retry in range(0, 10):
            self.set_h2scan_in_cli_mode()

            if (retry % 1) == 0:
                self.ledcontrol.led_on()
            if (retry % 2) == 0:
                self.ledcontrol.led_off()

            self.console.open(com=self.COMPORT, baudrate=19200)
            try:
                self.ask_h2scan_prompt(nbretry=3)
                time.sleep(1)
                self.console.ser.flushInput()
                self.set_h2scan_advanced()

                print 'Current H2Scan firmware version: %s' % (self.get_h2scan_version())
                return True
            except:
                self.console.close()

        return False
